﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.file;
using simpledb.buffer;
using simpledb.record;

namespace simpledb.index.btree
{
    /// <summary>
    /// An object that can format a page to look like an
    /// empty B-tree block.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class BTPageFormatter : PageFormatter
    {
        private TableInfo ti;
        private int flag;

        /// <summary>
        /// Creates a formatter for a new page of the
        /// specified B-tree index.
        /// </summary>
        /// <param name="ti">the index's metadata</param>
        /// <param name="flag">the page's initial flag value</param>
        public BTPageFormatter(TableInfo ti, int flag)
        {
            this.ti = ti;
            this.flag = flag;
        }

        /// <summary>
        /// Formats the page by initializing as many index-record slots
        /// as possible to have default values.
        /// Each integer field is given a value of 0, and
        /// each string field is given a value of "".
        /// The location that indicates the number of records
        /// in the page is also set to 0.
        /// </summary>
        /// <seealso cref="simpledb.buffer.PageFormatter.format"/>  
        public void format(Page page)
        {
            page.setInt(0, flag);
            page.setInt(Page.INT_SIZE, 0); // #record = 0;
            int recsize = ti.recordLength();
            for (int pos = 2 * Page.INT_SIZE; pos + recsize <= Page.BLOCK_SIZE; pos += recsize)
                makeDefaultRecord(page, pos);
        }

        private void makeDefaultRecord(Page page, int pos)
        {
            foreach (string fldname in ti.schema().fields())
            {
                int offset = ti.offset(fldname);
                if (ti.schema().type(fldname) == Schema.INTEGER)
                    page.setInt(pos + offset, 0);
                else
                    page.setString(pos + offset, "");
            }
        }
    }
}
