// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections;
using System.Collections.Generic;
using simpledb.file;

namespace simpledb.log
{
    /// <summary>
    ///  A class that provides the ability to move through the
    /// records of the log file in reverse order.
    /// </summary>
    /// <author>Edward Sciore</author>
    internal class LogIterator : IEnumerator<BasicLogRecord>
    {
        private Block blk;
        private Page pg = new Page();
        private Block currentblk;
        private int currentrec;

        /// <summary>
        /// Creates an iterator for the records in the log file,
        /// positioned after the last log record.
        /// This constructor is called exclusively by
        /// <see cref="LogMgr.GetEnumerator"/> .
        /// </summary>
        /// <param name="blk">The start Block for iterator.</param>
        public LogIterator(Block blk)
        {
            this.blk = blk;
            pg.read(blk);
            Reset();
        }

        /// <summary>
        ///  Determines if the current log record
        /// is the earliest record in the log file.
        /// </summary>
        /// <returns>true if there is an earlier record</returns>
        private bool hasNext()
        {
            return currentrec > 0 || currentblk.number() > 0;
        }

        /// <summary>
        ///  Moves to the next log block in reverse order,
        /// and positions it after the last record in that block.
        /// </summary>
        private void moveToNextBlock()
        {
            currentblk = new Block(currentblk.fileName(), currentblk.number() - 1);
            pg.read(currentblk);
            currentrec = pg.getInt(LogMgr.LAST_POS);
        }

        /// <summary>
        /// Gets the current log record.
        /// </summary>
        public BasicLogRecord Current
        {
            get
            {
                return new BasicLogRecord(pg, currentrec + Page.INT_SIZE);
            }
        }

        object IEnumerator.Current
        {
            get
            {
                return Current;
            }
        }

        /// <summary>
        /// Advances to the next log record in reverse order.
        /// </summary>
        /// <returns>
        /// true if successfully advanced, otherwise false
        /// </returns>
        public bool MoveNext()
        {
            if (!hasNext())
                return false;

            if (currentrec == 0)
                moveToNextBlock();
            currentrec = pg.getInt(currentrec);
            return true;
        }

        /// <summary>
        /// Sets the enumerator to its initial position.
        /// </summary>
        public void Reset()
        {
            currentblk = blk;
            currentrec = pg.getInt(LogMgr.LAST_POS);
        }

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
        }
    }
}