﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.query;

namespace simpledb.materialize
{
    /// <summary>
    /// The interface implemented by aggregation functions.
    /// Aggregation functions are used by the <i>groupby</i> operator.
    /// </summary>
    /// <author>Edward Sciore</author>
    public interface AggregationFn
    {
        /// <summary>
        ///  Uses the current record of the specified scan
        /// to be the first record in the group.
        /// </summary>
        /// <param name="s">the scan to aggregate over.</param> 
        void processFirst(Scan s);

        /// <summary>
        ///  Uses the current record of the specified scan
        /// to be the next record in the group.
        /// </summary>
        /// <param name="s">the scan to aggregate over.</param> 
        void processNext(Scan s);

        /// <summary>
        ///  Returns the name of the new aggregation field.
        /// </summary>
        /// <returns>the name of the new aggregation field</returns>
        string fieldName();

        /// <summary>
        ///  Returns the computed aggregation value.
        /// </summary>
        /// <returns>the computed aggregation value</returns> 
        Constant value();
    }
}
