﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.query;

namespace simpledb.materialize
{
    /// <summary>
    /// The <i>count</i> aggregation function.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class CountFn
    {
        private string fldname;
        private int count;

        /// <summary>
        ///  Creates a count aggregation function for the specified field.   
        /// </summary>
        /// <param name="fldname">the name of the aggregated field</param>  
        public CountFn(string fldname)
        {
            this.fldname = fldname;
        }

        /// <summary>
        /// Starts a new count.
        /// Since SimpleDB does not support null values,
        /// every record will be counted,
        /// regardless of the field.
        /// The current count is thus set to 1.
        /// </summary>
        /// <seealso cref="simpledb.materialize.AggregationFn.processFirst"/> 
        public void processFirst(Scan s)
        {
            count = 1;
        }

        /// <summary>
        ///  Since SimpleDB does not support null values,
        /// this method always increments the count,
        /// regardless of the field.  
        /// </summary>
        /// <seealso cref="simpledb.materialize.AggregationFn.processNext"/>
        public void processNext(Scan s)
        {
            count++;
        }

        /// <summary>
        ///  Returns the field's name, prepended by "countof".
        /// </summary>
        /// <seealso cref="simpledb.materialize.AggregationFn.fieldName"/> 
        public string fieldName()
        {
            return "countof" + fldname;
        }

        /// <summary>
        ///  Returns the current count.
        /// </summary>
        /// <seealso cref="simpledb.materialize.AggregationFn.value"/> 
        public Constant value()
        {
            return new IntConstant(count);
        }
    }
}
