﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.query;

namespace simpledb.materialize
{
    /// <summary>
    ///  The <i>max</i> aggregation function.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class MaxFn
    {
        private string fldname;
        private Constant val;

        /// <summary>
        ///  Creates a max aggregation function for the specified field.
        /// </summary>
        /// <param name="fldname">the name of the aggregated field</param>  
        public MaxFn(string fldname)
        {
            this.fldname = fldname;
        }

        /// <summary>
        /// Starts a new maximum to be the 
        /// field value in the current record.
        /// </summary>
        /// <seealso cref="simpledb.materialize.AggregationFn.processFirst"/> 
        public void processFirst(Scan s)
        {
            val = s.getVal(fldname);
        }

        /// <summary>
        /// Replaces the current maximum by the field value
        /// in the current record, if it is higher.
        /// </summary>
        /// <seealso cref="simpledb.materialize.AggregationFn.processNext"/> 
        public void processNext(Scan s)
        {
            Constant newval = s.getVal(fldname);
            if (newval.CompareTo(val) > 0)
                val = newval;
        }

        /// <summary>
        /// Returns the field's name, prepended by "maxof".
        /// </summary>
        /// <seealso cref="simpledb.materialize.AggregationFn.fieldName"/> 
        public string fieldName()
        {
            return "maxof" + fldname;
        }

        /// <summary>
        /// Returns the current maximum.
        /// </summary>
        /// <seealso cref="simpledb.materialize.AggregationFn.value"/> 
        public Constant value()
        {
            return val;
        }
    }
}
