﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.query;

namespace simpledb.materialize
{
    /// <summary>
    /// A comparator for scans.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class RecordComparator
    {
        private List<string> fields;

        /// <summary>
        ///  Creates a comparator using the specified fields,
        /// using the ordering implied by its iterator.
        /// </summary>
        /// <param name="fields">a list of field names</param> 
        public RecordComparator(List<string> fields)
        {
            this.fields = fields;
        }

        /// <summary>
        /// Compares the current records of the two specified scans.
        /// The sort fields are considered in turn.
        /// When a field is encountered for which the records have
        /// different values, those values are used as the result
        /// of the comparison.
        /// If the two records have the same values for all
        /// sort fields, then the method returns 0.
        /// </summary>
        /// <param name="s1">the first scan</param> 
        /// <param name="s2"> the second scan</param>
        /// <returns>the result of comparing each scan's current record according to the field list</returns> 
        public int compare(Scan s1, Scan s2)
        {
            foreach (string fldname in fields)
            {
                Constant val1 = s1.getVal(fldname);
                Constant val2 = s2.getVal(fldname);
                int result = val1.CompareTo(val2);
                if (result != 0)
                    return result;
            }
            return 0;
        }
    }
}
