// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm


namespace simpledb.metadata
{
    /// <summary>
    /// Holds three pieces of statistical information about a table:
    /// the number of blocks, the number of records,
    /// and the number of distinct values for each field. 
    /// </summary>
    /// <author>Edward Sciore</author> 
    public class StatInfo
    {
        private int numBlocks;
        private int numRecs;

        /// <summary>
        /// Creates a StatInfo object.
        /// Note that the number of distinct values is not
        /// passed into the constructor.
        /// The object fakes this value.
        /// </summary>
        /// <param name="numblocks">the number of blocks in the table</param>
        /// <param name="numrecs">the number of records in the table</param>
        public StatInfo(int numblocks, int numrecs)
        {
            this.numBlocks = numblocks;
            this.numRecs = numrecs;
        }

        /// <summary>
        /// Returns the estimated number of blocks in the table.
        /// </summary>
        /// <returns>the estimated number of blocks in the table</returns>
        public int blocksAccessed()
        {
            return numBlocks;
        }

        /// <summary>
        /// Returns the estimated number of records in the table.
        /// </summary>
        /// <returns>the estimated number of records in the table</returns>
        public int recordsOutput()
        {
            return numRecs;
        }

        /// <summary>
        /// Returns the estimated number of distinct values
        /// for the specified field.
        /// In actuality, this estimate is a complete guess.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>a guess as to the number of distinct field values</returns>
        public int distinctValues(string fldname)
        {
            return 1 + (numRecs / 3);
        }
    }
}
