﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.query;

namespace simpledb.parse
{
    /// <summary>
    /// Data for the SQL <i>insert</i> statement.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class InsertData
    {
        private string tblname;
        private List<string> flds;
        private List<Constant> i_vals;

        /// <summary>
        /// Saves the table name and the field and value lists.
        /// </summary>
        /// <param name="tblname"></param>
        /// <param name="flds"></param>
        /// <param name="vals"></param>
        public InsertData(string tblname, List<string> flds, List<Constant> vals)
        {
            this.tblname = tblname;
            this.flds = flds;
            this.i_vals = vals;
        }

        /// <summary>
        /// Returns the name of the affected table.
        /// </summary>
        /// <returns>the name of the affected table</returns>
        public string tableName()
        {
            return tblname;
        }

        /// <summary>
        /// Returns a list of fields for which
        /// values will be specified in the new record.
        /// </summary>
        /// <returns>a list of field names</returns>
        public List<string> fields()
        {
            return flds;
        }

        /// <summary>
        /// Returns a list of values for the specified fields.
        /// There is a one-one correspondence between this
        /// list of values and the list of fields.
        /// </summary>
        /// <returns> a list of Constant values.</returns>
        public List<Constant> vals()
        {
            return i_vals;
        }
    }
}
