﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.query;

namespace simpledb.parse
{
    /// <summary>
    /// Data for the SQL <i>select</i> statement.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class QueryData
    {
        private List<string> i_fields;
        private List<string> i_tables;
        private Predicate i_pred;

        /// <summary>
        /// Saves the field and table list and predicate.
        /// </summary>
        /// <param name="fields"></param>
        /// <param name="tables"></param>
        /// <param name="pred"></param>
        public QueryData(List<string> fields, List<string> tables, Predicate pred)
        {
            this.i_fields = fields;
            this.i_tables = tables;
            this.i_pred = pred;
        }

        /// <summary>
        /// Returns the fields mentioned in the select clause.
        /// </summary>
        /// <returns>a collection of field names</returns>
        public List<string> fields()
        {
            return i_fields;
        }

        /// <summary>
        /// Returns the tables mentioned in the from clause.
        /// </summary>
        /// <returns>a collection of table names</returns>
        public List<string> tables()
        {
            return i_tables;
        }

        /// <summary>
        /// Returns the predicate that describes which
        /// records should be in the output table.
        /// </summary>
        /// <returns>the query predicate</returns>
        public Predicate pred()
        {
            return i_pred;
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            string result = "select ";
            foreach (string fldname in i_fields)
                result += fldname + ", ";
            result = result.Substring(0, result.Length - 2); //remove final comma
            result += " from ";
            foreach (string tblname in i_tables)
                result += tblname + ", ";
            result = result.Substring(0, result.Length - 2); //remove final comma
            string predstring = i_pred.ToString();
            if (!predstring.Equals(""))
                result += " where " + predstring;
            return result;
        }
    }
}
