﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.tx;
using simpledb.parse;

namespace simpledb.planner
{
    /// <summary>
    /// The interface implemented by the planners
    /// for SQL insert, delete, and modify statements.
    /// </summary>
    /// <author>Edward Sciore</author>
    public interface UpdatePlanner
    {
        /// <summary>
        /// Executes the specified insert statement, and
        /// returns the number of affected records.
        /// </summary>
        /// <param name="data">the parsed representation of the insert statement</param>
        /// <param name="tx">the calling transaction</param>
        /// <returns>the number of affected records</returns>
        int executeInsert(InsertData data, Transaction tx);

        /// <summary>
        /// Executes the specified delete statement, and
        /// returns the number of affected records.
        /// </summary>
        /// <param name="data">the parsed representation of the delete statement</param>
        /// <param name="tx">the calling transaction</param>
        /// <returns>the number of affected records</returns>
        int executeDelete(DeleteData data, Transaction tx);

        /// <summary>
        /// Executes the specified modify statement, and
        /// returns the number of affected records.
        /// </summary>
        /// <param name="data">the parsed representation of the modify statement</param>
        /// <param name="tx">the calling transaction</param>
        /// <returns>number of affected records</returns>
        int executeModify(ModifyData data, Transaction tx);

        /// <summary>
        /// Executes the specified create table statement, and
        /// returns the number of affected records.
        /// </summary>
        /// <param name="data">the parsed representation of the create table statement</param>
        /// <param name="tx">the calling transaction</param>
        /// <returns>the number of affected records</returns>
        int executeCreateTable(CreateTableData data, Transaction tx);

        /// <summary>
        /// Executes the specified create view statement, and
        /// returns the number of affected records.
        /// </summary>
        /// <param name="data">the parsed representation of the create view statement</param>
        /// <param name="tx">the calling transaction</param>
        /// <returns>the number of affected records</returns>
        int executeCreateView(CreateViewData data, Transaction tx);

        /// <summary>
        /// Executes the specified create index statement, and
        /// returns the number of affected records.
        /// </summary>
        /// <param name="data">the parsed representation of the create index statement</param>
        /// <param name="tx">the calling transaction</param>
        /// <returns>the number of affected records</returns>
        int executeCreateIndex(CreateIndexData data, Transaction tx);
    }
}
