﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    ///  An expression consisting entirely of a single constant.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class ConstantExpression : Expression
    {
        private Constant val;

        /// <summary>
        /// Creates a new expression by wrapping a constant.
        /// </summary>
        /// <param name="c">the constant</param>  
        public ConstantExpression(Constant c)
        {
            val = c;
        }

        /// <summary>
        ///  Returns true.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.isConstant"/> 
        public bool isConstant()
        {
            return true;
        }

        /// <summary>
        /// Returns false.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.isFieldName"/> 
        public bool isFieldName()
        {
            return false;
        }

        /// <summary>
        ///  Unwraps the constant and returns it.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.asConstant"/> 
        public Constant asConstant()
        {
            return val;
        }

        /// <summary>
        /// This method should never be called.
        /// Throws a InvalidCastException.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.asFieldName"/> 
        public string asFieldName()
        {
            throw new System.InvalidCastException(); 
        }

        /// <summary>
        /// Returns the constant, regardless of the scan.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.evaluate"/>
        public Constant evaluate(Scan s)
        {
            return val;
        }

        /// <summary>
        ///  Returns true, because a constant applies to any schema.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.appliesTo"/> 
        public bool appliesTo(Schema sch)
        {
            return true;
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return val.ToString();
        }
    }
}
