﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    ///  An expression consisting entirely of a single field.
    /// </summary>
    /// /// <author>Edward Sciore</author>
    public class FieldNameExpression : Expression
    {
        private string fldname;

        /// <summary>
        ///  Creates a new expression by wrapping a field.
        /// </summary>
        /// <param name="fldname">the name of the wrapped field</param>  
        public FieldNameExpression(string fldname)
        {
            this.fldname = fldname;
        }

        /// <summary>
        ///  Returns false.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.isConstant"/> 
        public bool isConstant()
        {
            return false;
        }

        /// <summary>
        /// Returns true.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.isFieldName"/> 
        public bool isFieldName()
        {
            return true;
        }

        /// <summary>
        /// This method should never be called.
        /// Throws a InvalidCastException.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.asConstant"/> 
        public Constant asConstant()
        {
            throw new System.InvalidCastException();
        }

        /// <summary>
        /// Unwraps the field name and returns it.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.asFieldName"/> 
        public string asFieldName()
        {
            return fldname;
        }

        /// <summary>
        /// Evaluates the field by getting its value in the scan.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.evaluate"/> 
        public Constant evaluate(Scan s)
        {
            return s.getVal(fldname);
        }

        /// <summary>
        /// Returns true if the field is in the specified schema.
        /// </summary>
        /// <seealso cref="simpledb.query.Expression.appliesTo"/> 
        public bool appliesTo(Schema sch)
        {
            return sch.hasField(fldname);
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return fldname;
        }
    }
}
