﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    /// The interface implemented by each query plan.
    /// There is a Plan class for each relational algebra operator.
    /// </summary> 
    /// <author>Edward Sciore</author>
    public interface Plan
    {
        /// <summary>
        /// Opens a scan corresponding to this plan.
        /// The scan will be positioned before its first record.
        /// </summary>
        /// <returns>a scan</returns>
        Scan open();

        /// <summary>
        /// Returns an estimate of the number of block accesses
        /// that will occur when the scan is read to completion.
        /// </summary>
        /// <returns>the estimated number of block accesses</returns> 
        int blocksAccessed();

        /// <summary>
        /// Returns an estimate of the number of records
        /// in the query's output table.
        /// </summary>
        /// <returns>the estimated number of output records</returns>
        int recordsOutput();

        /// <summary>
        ///  /// Returns an estimate of the number of distinct values
        /// for the specified field in the query's output table.
        /// </summary>
        /// <param name="fldname">the name of a field</param>  
        /// <returns>the estimated number of distinct field values in the output</returns>
        int distinctValues(string fldname);

        /// <summary>
        ///  Returns the schema of the query.
        /// </summary>
        /// <returns>the query's schema</returns> 
        Schema schema();
    }
}
