﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System.Collections.Generic;
using simpledb.record;

namespace simpledb.query
{
    /// <summary>
    /// The Plan class corresponding to the <i>project</i>
    /// relational algebra operator.
    /// </summary> 
    /// <author>Edward Sciore</author>
    public class ProjectPlan : Plan
    {
        private Plan p;
        private Schema sch = new Schema();

        /// <summary>
        ///  Creates a new project node in the query tree,
        /// having the specified subquery and field list.
        /// </summary>
        /// <param name="p">the subquery</param>
        /// <param name="fieldlist">the list of fields</param>
        public ProjectPlan(Plan p, ICollection<string> fieldlist)
        {
            this.p = p;
            foreach (string fldname in fieldlist)
                sch.add(fldname, p.schema());
        }

        /// <summary>
        /// Creates a project scan for this query.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.open"/> 
        public Scan open()
        {
            Scan s = p.open();
            return new ProjectScan(s, sch.fields());
        }

        /// <summary>
        /// Estimates the number of block accesses in the projection,
        /// which is the same as in the underlying query.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.blocksAccessed"/> 
        public int blocksAccessed()
        {
            return p.blocksAccessed();
        }

        /// <summary>
        /// Estimates the number of output records in the projection,
        /// which is the same as in the underlying query.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.recordsOutput"/> 
        public int recordsOutput()
        {
            return p.recordsOutput();
        }

        /// <summary>
        /// Estimates the number of distinct field values
        /// in the projection,
        /// which is the same as in the underlying query.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.distinctValues"/>
        public int distinctValues(string fldname)
        {
            return p.distinctValues(fldname);
        }

        /// <summary>
        ///  Returns the schema of the projection,
        /// which is taken from the field list.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.schema"/>
        public Schema schema()
        {
            return sch;
        }
    }
}
