﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using System;
using System.Collections.Generic;

namespace simpledb.query
{
    /// <summary>
    /// The scan class corresponding to the <i>project</i> relational
    /// algebra operator.
    /// All methods except hasField delegate their work to the
    /// underlying scan.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class ProjectScan : Scan
    {
        private Scan s;
        private ICollection<string> fieldlist;

        /// <summary>
        /// Creates a project scan having the specified
        /// underlying scan and field list. 
        /// </summary>
        /// <param name="s">the underlying scan</param>
        /// <param name="fieldlist">the list of field names</param>
        public ProjectScan(Scan s, ICollection<string> fieldlist)
        {
            this.s = s;
            this.fieldlist = fieldlist;
        }

        /// <summary>
        ///  Positions the scan before its first record.
        /// </summary>
        public void beforeFirst()
        {
            s.beforeFirst();
        }

        /// <summary>
        /// Moves the scan to the next record.
        /// </summary>
        /// <returns>false if there is no next record</returns> 
        public bool next()
        {
            return s.next();
        }

        /// <summary>
        ///  Closes the scan and its subscans, if any. 
        /// </summary>
        public void close()
        {
            s.close();
        }

        /// <summary>
        /// Returns the value of the specified integer field 
        /// in the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the field's integer value in the current record</returns>
        public Constant getVal(string fldname)
        {
            if (hasField(fldname))
                return s.getVal(fldname);
            else
                throw new Exception("field " + fldname + " not found.");
        }

        /// <summary>
        /// Returns the value of the specified integer field 
        /// in the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the field's integer value in the current record</returns>
        public int getInt(string fldname)
        {
            if (hasField(fldname))
                return s.getInt(fldname);
            else
                throw new Exception("field " + fldname + " not found.");
        }

        /// <summary>
        ///  Returns the value of the specified string field 
        /// in the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the field's string value in the current record</returns>
        public string getString(string fldname)
        {
            if (hasField(fldname))
                return s.getString(fldname);
            else
                throw new Exception(" field " + fldname + " not found.");
        }
        /// <summary>
        /// Returns true if the specified field
        /// is in the projection list.
        /// </summary> 
        /// <seealso cref="simpledb.query.Scan.hasField"/> 
        public bool hasField(string fldname)
        {
            return fieldlist.Contains(fldname);
        }
    }
}
