﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm


namespace simpledb.query
{
    /// <summary>
    /// The interface will be implemented by each query scan.
    /// There is a Scan class for each relational
    /// algebra operator.
    /// </summary> 
    /// <author>Edward Sciore</author>
    public interface Scan
    {
        /// <summary>
        ///  Positions the scan before its first record.
        /// </summary>
        void beforeFirst();

        /// <summary>
        /// Moves the scan to the next record.
        /// </summary>
        /// <returns>false if there is no next record</returns> 
        bool next();

        /// <summary>
        ///  Closes the scan and its subscans, if any. 
        /// </summary>
        void close();

        /// <summary>
        /// Returns the value of the specified field in the current record.
        /// The value is expressed as a Constant.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the value of that field, expressed as a Constant.</returns>
        Constant getVal(string fldname);

        /// <summary>
        /// Returns the value of the specified integer field 
        /// in the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the field's integer value in the current record</returns>
        int getInt(string fldname);

        /// <summary>
        ///  Returns the value of the specified string field 
        /// in the current record.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>the field's string value in the current record</returns>
        string getString(string fldname);

        /// <summary>
        /// Returns true if the scan has the specified field.
        /// </summary>
        /// <param name="fldname">the name of the field</param>
        /// <returns>true if the scan has that field</returns>
        bool hasField(string fldname);
    }
}
