﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.tx;
using simpledb.record;
using simpledb.metadata;
using simpledb.server;

namespace simpledb.query
{
    /// <summary>
    /// The Plan class corresponding to a table.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class TablePlan : Plan
    {
        private Transaction tx;
        private TableInfo ti;
        private StatInfo si;

        /// <summary>
        ///  Creates a leaf node in the query tree corresponding
        /// to the specified table.
        /// </summary>
        /// <param name="tblname">the name of the table</param>
        /// <param name="tx">the calling transaction</param>
        public TablePlan(string tblname, Transaction tx)
        {
            this.tx = tx;
            ti = SimpleDB.mdMgr().getTableInfo(tblname, tx);
            si = SimpleDB.mdMgr().getStatInfo(tblname, ti, tx);
        }

        /// <summary>
        ///  Creates a table scan for this query.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.open"/> 
        public Scan open()
        {
            return new TableScan(ti, tx);
        }

        /// <summary>
        ///  Estimates the number of block accesses for the table,
        /// which is obtainable from the statistics manager.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.blocksAccessed"/> 
        public int blocksAccessed()
        {
            return si.blocksAccessed();
        }

        /// <summary>
        ///  /// Estimates the number of records in the table,
        /// which is obtainable from the statistics manager.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.recordsOutput"/> 
        public int recordsOutput()
        {
            return si.recordsOutput();
        }

        /// <summary>
        ///  Estimates the number of distinct field values in the table,
        /// which is obtainable from the statistics manager.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.distinctValues"/> 
        public int distinctValues(string fldname)
        {
            return si.distinctValues(fldname);
        }

        /// <summary>
        /// Determines the schema of the table,
        /// which is obtainable from the catalog manager.
        /// </summary>
        /// <seealso cref="simpledb.query.Plan.schema"/> 
        public Schema schema()
        {
            return ti.schema();
        }
    }
}
