﻿// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm


namespace simpledb.record
{
    /// <summary>
    /// An identifier for a record within a file.
    /// A RID consists of the block number in the file,
    /// and the ID of the record in that block.
    /// </summary>
    /// <author>Edward Sciore</author>
    public class RID
    {
        private int blknum;
        private int id;

        /// <summary>
        /// Creates a RID for the record having the
        /// specified ID in the specified block.
        /// </summary>
        /// <param name="blknum">the block number where the record lives</param>
        /// <param name="id">the record's ID</param>
        public RID(int blknum, int id)
        {
            this.blknum = blknum;
            this.id = id;
        }

        /// <summary>
        /// Returns the block number associated with this RID.
        /// </summary>
        /// <returns>the block number</returns>
        public int blockNumber()
        {
            return blknum;
        }

        /// <summary>
        /// Returns the ID associated with this RID.
        /// </summary>
        /// <returns>the ID</returns>
        public int Id()
        {
            return id;
        }

        /// <summary>
        /// Determines whether the specified <see cref="System.Object"/> is equal to this instance.
        /// </summary>
        /// <param name="obj">The <see cref="System.Object"/> to compare with this instance.</param>
        /// <returns>
        /// 	<c>true</c> if the specified <see cref="System.Object"/> is equal to this instance; otherwise, <c>false</c>.
        /// </returns>
        /// <exception cref="T:System.NullReferenceException">
        /// The <paramref name="obj"/> parameter is null.
        /// </exception>
        public override bool Equals(object obj)
        {
            RID r = (RID)obj;
            return blknum == r.blknum && id == r.id;
        }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return "[" + blknum + "," + id + "]";
        }

        /// <summary>
        /// Serves as a hash function for a particular type. 
        /// </summary>
        /// <returns>A hash code for the current Block object.</returns>
        public override int GetHashCode()
        {
            return ToString().GetHashCode();
        }
    }
}
