// migration from java to .NET by SimpleDB# group, 
// http://cs.tju.edu.cn/faculty/zhangkl/projects/simpledbsharp/index.htm

using simpledb.log;

namespace simpledb.tx.recovery
{
    /// <summary>
    /// The StartRecord log record.
    /// </summary>
    /// <author>Edward Sciore</author> 
    class StartRecord : LogRecord
    {
        private int txnum;

        /// <summary>
        /// Creates a new start log record for the specified transaction.
        /// </summary>
        /// <param name="txnum">the ID of the specified transaction</param>
        public StartRecord(int txnum)
        {
            this.txnum = txnum;
        }

        /// <summary>
        /// Creates a log record by reading one other value from the log.
        /// </summary>
        /// <param name="rec">the basic log record</param>
        public StartRecord(BasicLogRecord rec)
        {
            txnum = rec.nextInt();
        }

        /// <summary>
        /// Writes a start record to the log.
        /// This log record contains the START operator,
        /// followed by the transaction id.
        /// </summary>
        /// <returns>the LSN of the last log value</returns>
        public override int writeToLog()
        {
            object[] rec = new object[] { START, txnum };
            return logMgr.append(rec);
        }

        /// <summary>
        /// returns the log record's type
        /// </summary>
        /// <returns>the log record's type</returns>
        public override int op()
        {
            return START;
        }

        /// <summary>
        /// Returns the transaction id stored with
        /// the log record.
        /// </summary>
        /// <returns>the log record's transaction id</returns>
        public override int txNumber()
        {
            return txnum;
        }

        /// <summary>
        /// Does nothing, because a start record
        /// contains no undo information 
        /// </summary>
        public override void undo(int txnum) { }

        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return "<START " + txnum + ">";
        }
    }
}
