letter		::=	[a-zA-Z];
digit		::=	[0-9];
string_regular	::=	[^\'\r\n\\];
escape_char	::=	[^\r\n];
whitespace	::=	[ \r\n\t];

A::="A"|"a";  B::="B"|"b";  C::="C"|"c";  D::="D"|"d";
E::="E"|"e";  F::="F"|"f";  G::="G"|"g";  H::="H"|"h";
I::="I"|"i";  J::="J"|"j";  K::="K"|"k";  L::="L"|"l";
M::="M"|"m";  N::="N"|"n";  O::="O"|"o";  P::="P"|"p";
Q::="Q"|"q";  R::="R"|"r";  S::="S"|"s";  T::="T"|"t";
U::="U"|"u";  V::="V"|"v";  W::="W"|"w";  X::="X"|"x";
Y::="Y"|"y";  Z::="Z"|"z";

DELIM		::=	whitespace+;

KW_SELECT	::=	S E L E C T;
KW_FROM		::=	F R O M;
KW_WHERE	::=	W H E R E;
KW_AND		::=	A N D;
KW_INSERT	::=	I N S E R T;
KW_INTO		::=	I N T O;
KW_VALUES	::=	V A L U E S;
KW_DELETE	::=	D E L E T E;
KW_UPDATE	::=	U P D A T E;
KW_SET		::=	S E T;
KW_CREATE	::=	C R E A T E;
KW_TABLE	::=	T A B L E;
KW_INT		::=	I N T;
KW_VARCHAR	::=	V A R C H A R;
KW_VIEW		::=	V I E W;
KW_AS		::=	A S;
KW_INDEX	::=	I N D E X;
KW_ON		::=	O N;

IDENTIFIER	::=	(letter|"_") (letter|digit|"_")* ;
INTEGER		::=	("+"|"-")? digit+ ;
STRING		::=	"'" (string_regular | "\\" escape_char)*  "'";

OP_COMMA	::=	",";
OP_P_L		::=	"(";
OP_P_R		::=	")";
OP_EQ		::=	"=";


token {
	KW_SELECT,	KW_FROM,	KW_WHERE,	KW_AND,
	KW_INSERT,	KW_INTO,	KW_VALUES,	KW_DELETE,
	KW_UPDATE,	KW_SET,		KW_CREATE,	KW_TABLE,
	KW_INT,		KW_VARCHAR,	KW_VIEW,	KW_AS,
	KW_INDEX,	KW_ON,
	IDENTIFIER,	INTEGER,	STRING,
	OP_COMMA,	OP_P_L,		OP_P_R,		OP_EQ,
	~DELIM
}

%% //////////////////////////// bnf /////////////////////////////

program  ::=  query		## {program1();}
program  ::=  insert		## {program2();}
program  ::=  delete		## {program3();}
program  ::=  update		## {program4();}
program  ::=  create		## {program5();}

query  ::= KW_SELECT  identifier-list  KW_FROM  identifier-list  where-clause	##{query1();}
query  ::= KW_SELECT  identifier-list  KW_FROM  identifier-list			##{query2();}

identifier-list  ::=  identifier-list  OP_COMMA  identifier			##{identifier_list1();}
identifier-list  ::=  identifier						##{identifier_list2();}

identifier  ::=  IDENTIFIER	##{identifier1();}
integer     ::=  INTEGER	##{integer1();}
string      ::=  STRING         ##{string1();}

where-clause  ::=  KW_WHERE  predicate		##{where_clause1();}

predicate  ::=  predicate  KW_AND  term		##{predicate1();}
predicate  ::=  term				##{predicate2();}
term  ::=  expression  OP_EQ  expression	##{term1();}

expression  ::=  identifier			##{expression1();}
expression  ::=  constant			##{expression2();}

constant  ::=  string				##{constant1();}
constant  ::=  integer				##{constant2();}

insert  ::=  KW_INSERT  KW_INTO  identifier  OP_P_L  identifier-list  OP_P_R  KW_VALUES  OP_P_L  constant-list  OP_P_R	##{insert1();}

constant-list  ::=  constant-list  OP_COMMA  constant	##{constant_list1();}
constant-list  ::=  constant				##{constant_list2();}

delete  ::=  KW_DELETE  KW_FROM  identifier  where-clause	##{delete1();}
delete  ::=  KW_DELETE  KW_FROM  identifier			##{delete2();}

update  ::=  KW_UPDATE  identifier  KW_SET  identifier  OP_EQ  expression  where-clause		##{update1();}
update  ::=  KW_UPDATE  identifier  KW_SET  identifier  OP_EQ  expression			##{update2();}

create  ::=  create-table	##{create1();}
create  ::=  create-view	##{create2();}
create  ::=  create-index	##{create3();}

create-table  ::=  KW_CREATE  KW_TABLE  identifier  OP_P_L  field-def-list  OP_P_R	##{create_table1();}

field-def-list  ::=  field-def-list  OP_COMMA  field-def	##{field_def_list1();}
field-def-list  ::=  field-def					##{field_def_list2();}

field-def  ::=  identifier  KW_INT					##{field_def1();}
field-def  ::=  identifier  KW_VARCHAR  OP_P_L  integer  OP_P_R		##{field_def2();}

create-view  ::=  KW_CREATE  KW_VIEW  identifier  KW_AS  query		##{create_view1();}
create-index  ::=  KW_CREATE  KW_INDEX  identifier  KW_ON  identifier  OP_P_L  identifier  OP_P_R	##{create_index1();}


%%
